/**!
* tippy.js v5.0.0-beta.0
* (c) 2017-2019 atomiks
* MIT License
*/
import { _ as _extends, h as hasOwnProperty, p as preserveInvocation, F as removeProperties, l as getValueAtIndexOrReturn, N as NON_UPDATEABLE_PROPS } from '../../esm/tippy.chunk.js';
import { a as throwErrorWhen, t as tippy } from '../../esm/tippy.chunk2.js';
import 'popper.js';

/**
 * Creates a delegate instance that controls the creation of tippy instances
 * for child elements (`target` CSS selector).
 */
function delegate(targets, props) {
  if (process.env.NODE_ENV !== "production") {
    throwErrorWhen(!props || !props.target, 'You must specify a `target` prop indicating the CSS selector string ' + 'matching the target elements that should receive a tippy.');
  }

  var listeners = [];
  var childTippyInstances = [];
  var target = props.target;
  delete props.target; // The user needs to specify their own enhanced tippy function to use extra
  // props
  // @ts-ignore

  var tippyConstructor = delegate.tippy || tippy;
  var returnValue = tippyConstructor(targets, props);

  function onTrigger(event) {
    if (event.target) {
      var targetNode = event.target.closest(target);

      if (targetNode) {
        var instance = tippyConstructor(targetNode, _extends({}, props, {
          showOnCreate: true
        }));

        if (instance) {
          childTippyInstances = childTippyInstances.concat(instance);
        }
      }
    }
  }

  function on(element, eventType, listener, options) {
    if (options === void 0) {
      options = false;
    }

    element.addEventListener(eventType, listener, options);
    listeners.push({
      element: element,
      eventType: eventType,
      listener: listener,
      options: options
    });
  }

  function addEventListeners(instance) {
    var reference = instance.reference;
    instance.props.trigger.trim().split(' ').forEach(function (eventType) {
      switch (eventType) {
        case 'mouseenter':
          {
            on(reference, 'mouseover', onTrigger);
            break;
          }

        case 'focus':
          {
            on(reference, 'focusin', onTrigger);
            break;
          }

        case 'click':
          {
            on(reference, 'click', onTrigger);
          }
      }
    });
  }

  function removeEventListeners(listeners) {
    listeners.forEach(function (_ref) {
      var element = _ref.element,
          eventType = _ref.eventType,
          listener = _ref.listener,
          options = _ref.options;
      element.removeEventListener(eventType, listener, options);
    });
    listeners = [];
  }

  function applyMutations(instance) {
    var originalDestroy = instance.destroy;

    instance.destroy = function (shouldDestroyChildInstances) {
      if (shouldDestroyChildInstances === void 0) {
        shouldDestroyChildInstances = true;
      }

      if (shouldDestroyChildInstances) {
        childTippyInstances.forEach(function (instance) {
          instance.destroy();
        });
      }

      childTippyInstances = [];
      removeEventListeners(listeners);
      originalDestroy();
    };

    addEventListeners(instance);
    instance.setProps({
      trigger: 'manual'
    });
  }

  var instances = [].concat(returnValue);
  instances.forEach(applyMutations);
  return returnValue;
}

/**
 * Re-uses a single tippy element for many different tippy instances.
 * Replaces v4's `tippy.group()`.
 */
function createSingleton(tippyInstances, optionalProps) {
  if (optionalProps === void 0) {
    optionalProps = {
      delay: 0
    };
  }

  if (process.env.NODE_ENV !== "production") {
    if (!Array.isArray(tippyInstances)) {
      throwErrorWhen(!tippyInstances, 'First argument to `createSingleton()` must be an array of tippy ' + 'instances. The passed value was `' + tippyInstances + '`');
      throwErrorWhen( // @ts-ignore
      tippyInstances.reference && tippyInstances.reference._tippy, 'First argument to `createSingleton()` must be an *array* of tippy ' + 'instances. The passed value was a *single* tippy instance.');
    }

    var isAnyInstancePartOfExistingSingleton = tippyInstances.some(function (instance) {
      return hasOwnProperty(instance, '__singleton__');
    });
    throwErrorWhen(isAnyInstancePartOfExistingSingleton, 'The passed tippy instance(s) are already part of an existing ' + 'singleton instance. Make sure you destroy the previous singleton ' + 'before calling `createSingleton()` again.');
    tippyInstances.forEach(function (instance) {
      Object.defineProperty(instance, '__singleton__', {
        value: true,
        enumerable: false,
        configurable: true
      });
    });
  } // The user needs to specify their own enhanced tippy function to use extra
  // props
  // @ts-ignore


  var tippyConstructor = createSingleton.tippy || tippy;
  var singletonInstance = tippyConstructor(document.createElement('div'));
  var _optionalProps = optionalProps,
      delay = _optionalProps.delay;
  var showTimeout;
  var hideTimeout;

  function clearTimeouts() {
    clearTimeout(showTimeout);
    clearTimeout(hideTimeout);
  }

  tippyInstances.forEach(function (instance) {
    instance.__originalProps__ = {
      delay: instance.props.delay,
      onTrigger: instance.props.onTrigger,
      onUntrigger: instance.props.onUntrigger // To prevent bugs with `hideOnClick`, we need to let the original tippy
      // instance also go through its lifecycle (i.e. be mounted to the DOM as
      // well). To prevent it from being seen/overlayed over the singleton
      // tippy, we can set its opacity to 0

    };
    instance.popper.style.opacity = '0';
    instance.__originalClearDelayTimeouts__ = instance.clearDelayTimeouts;

    instance.clearDelayTimeouts = function () {
      instance.__originalClearDelayTimeouts__();

      clearTimeouts();
    };

    instance.setProps({
      delay: 0,
      onTrigger: function onTrigger(_, event) {
        preserveInvocation(instance.__originalProps__.onTrigger, instance.props.onTrigger, [instance, event]);
        singletonInstance.setProps(removeProperties(instance.props, NON_UPDATEABLE_PROPS.concat('delay')));

        singletonInstance.reference.getBoundingClientRect = function () {
          return instance.reference.getBoundingClientRect();
        };

        clearTimeouts(); // Edge case: if the tippy is currently hiding (but still mounted and
        // visible due to its opacity), it will slide to the new reference
        // element but fully to fade out before fading back in.
        // Also, we need to ensure the `popper` element doesn't set its
        // `transitionDuration` to 0ms, so it can transition smoothly

        if (!singletonInstance.state.isVisible && singletonInstance.state.isMounted) {
          singletonInstance.show(undefined, false);
        } else {
          showTimeout = setTimeout(function () {
            singletonInstance.show();
          }, getValueAtIndexOrReturn(delay, 0, tippy.defaultProps.delay));
        }
      },
      onUntrigger: function onUntrigger(_, event) {
        preserveInvocation(instance.__originalProps__.onUntrigger, instance.props.onUntrigger, [instance, event]);
        clearTimeouts();
        hideTimeout = setTimeout(function () {
          singletonInstance.hide();
        }, getValueAtIndexOrReturn(delay, 1, tippy.defaultProps.delay));
      }
    }); // Ensure the lifecycle functions can be updated

    instance.__originalSetProps__ = instance.setProps;

    instance.setProps = function (partialProps) {
      if (partialProps.onTrigger) {
        instance.__originalProps__.onTrigger = partialProps.onTrigger;
      }

      if (partialProps.onUntrigger) {
        instance.__originalProps__.onUntrigger = partialProps.onUntrigger;
      }

      instance.__originalSetProps__(removeProperties(partialProps, ['delay', 'onTrigger', 'onUntrigger']));
    };
  });
  var originalSetProps = singletonInstance.setProps;

  singletonInstance.setProps = function (partialProps) {
    delay = partialProps.delay !== undefined ? partialProps.delay : delay;
    originalSetProps(partialProps);
  };

  var originalDestroy = singletonInstance.destroy;

  singletonInstance.destroy = function (shouldDestroyPassedInstances) {
    if (shouldDestroyPassedInstances === void 0) {
      shouldDestroyPassedInstances = true;
    }

    tippyInstances.forEach(function (instance) {
      if (instance.state.isDestroyed) {
        return;
      } // Restore the instances to their original state


      instance.clearDelayTimeouts = instance.__originalClearDelayTimeouts__;
      instance.setProps = instance.__originalSetProps__;
      instance.setProps(instance.__originalProps__);
      delete instance.__originalClearDelayTimeouts__;
      delete instance.__originalProps__;
      delete instance.__originalSetProps__;

      if (process.env.NODE_ENV !== "production") {
        delete instance.__singleton__;
      }

      if (shouldDestroyPassedInstances) {
        instance.destroy();
      }
    });
    originalDestroy();
  };

  return singletonInstance;
}

export { createSingleton, delegate };
//# sourceMappingURL=tippy-addons.js.map
