/**!
* tippy.js v5.0.0-beta.0
* (c) 2017-2019 atomiks
* MIT License
*/
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var __chunk_1 = require('../../cjs/tippy.chunk.js');

function copyStatics(to, from) {
  to.currentInput = from.currentInput;
  to.version = from.version;
  to.defaultProps = from.defaultProps;
  to.setDefaultProps = from.setDefaultProps;
  to.hideAll = from.hideAll;
  return to;
}

function enhance(tippyBase, propHOFs) {
  var tippyEnhanced = propHOFs.reduce(function (nextTippy, addProp) {
    return addProp(nextTippy);
  }, tippyBase);
  return copyStatics(tippyEnhanced, tippyBase);
}

function withFollowCursor(tippy) {
  return function (targets, optionalProps) {
    return tippy(targets, __chunk_1._extends({}, optionalProps, {
      onCreate: function onCreate(instance) {
        __chunk_1.preserveInvocation(optionalProps && optionalProps.onCreate, instance.props.onCreate, [instance]);

        if (process.env.NODE_ENV !== "production") {
          instance.__extraProps__.followCursor = true;
        }

        if (instance.props.followCursor) {
          applyFollowCursor(instance);
        }
      }
    }));
  };
}

function applyFollowCursor(instance) {
  var reference = instance.reference;
  var originalSetProps = instance.setProps;
  var lastMouseMoveEvent;
  var isPopperInstanceCreated = false;
  var wasTriggeredByFocus = false;

  function addListener() {
    document.addEventListener('mousemove', onMouseMove);
  }

  function removeListener() {
    document.removeEventListener('mousemove', onMouseMove);
  }

  function onMouseMove(event) {
    if (wasTriggeredByFocus || !event) {
      return;
    }

    var _lastMouseMoveEvent = lastMouseMoveEvent = event,
        clientX = _lastMouseMoveEvent.clientX,
        clientY = _lastMouseMoveEvent.clientY;

    if (!instance.popperInstance || !isPopperInstanceCreated) {
      return;
    } // If the instance is interactive, avoid updating the position unless it's
    // over the reference element


    var isCursorOverReference = __chunk_1.closestCallback(event.target, function (el) {
      return el === reference;
    });
    var rect = reference.getBoundingClientRect();
    var _ref = instance.props,
        followCursor = _ref.followCursor;
    var isHorizontal = followCursor === 'horizontal';
    var isVertical = followCursor === 'vertical';
    var isVerticalPlacement = __chunk_1.includes(['top', 'bottom'], __chunk_1.getBasePlacement(instance.state.currentPlacement)); // The virtual reference needs some size to prevent itself from overflowing

    var _getVirtualOffsets = __chunk_1.getVirtualOffsets(instance.popper, isVerticalPlacement),
        size = _getVirtualOffsets.size,
        x = _getVirtualOffsets.x,
        y = _getVirtualOffsets.y;

    if (isCursorOverReference || !instance.props.interactive) {
      instance.popperInstance.reference = {
        // These `client` values don't get used by Popper.js if they are 0
        clientWidth: 0,
        clientHeight: 0,
        getBoundingClientRect: function getBoundingClientRect() {
          return {
            width: isVerticalPlacement ? size : 0,
            height: isVerticalPlacement ? 0 : size,
            top: (isHorizontal ? rect.top : clientY) - y,
            bottom: (isHorizontal ? rect.bottom : clientY) + y,
            left: (isVertical ? rect.left : clientX) - x,
            right: (isVertical ? rect.right : clientX) + x
          };
        }
      };
      instance.popperInstance.update();
    } // IIFE context vs. ESM/CJS context


    var ci = // @ts-ignore
    // eslint-disable-next-line
    typeof tippy !== 'undefined' ? tippy.currentInput : __chunk_1.currentInput;

    if (ci.isTouch || followCursor === 'initial' && instance.state.isVisible) {
      removeListener();
    }
  }

  var _instance$props = instance.props,
      popperOptions = _instance$props.popperOptions,
      _onMount = _instance$props.onMount,
      _onTrigger = _instance$props.onTrigger,
      _onUntrigger = _instance$props.onUntrigger,
      _onHidden = _instance$props.onHidden; // Due to the virtual offsets normalization when using `followCursor`,
  // we need to use the opposite placement

  var placement = instance.props.placement;
  var normalizedPlacement = placement;

  function setNormalizedPlacement() {
    var shift = placement.split('-')[1];
    normalizedPlacement = instance.props.followCursor && shift ? placement.replace(shift, shift === 'start' ? 'end' : 'start') : placement;
    originalSetProps({
      placement: normalizedPlacement
    });
  }

  setNormalizedPlacement();

  function popperOnCreate(data) {
    __chunk_1.preserveInvocation(popperOptions && popperOptions.onCreate, instance.props.popperOptions.onCreate, [data]);
    isPopperInstanceCreated = true;
  }

  instance.setProps({
    popperOptions: __chunk_1._extends({}, popperOptions, {
      onCreate: popperOnCreate
    }),
    onMount: function onMount(instance) {
      __chunk_1.preserveInvocation(_onMount, instance.props.onMount, [instance]); // Popper's scroll listeners make sense for `true`, where the cursor
      // follows both axes. TODO: somehow keep scroll listeners for vertical
      // scrolling for "vertical", and horizontal scrolling for "horizontal".

      if (!wasTriggeredByFocus && instance.props.followCursor !== true) {
        instance.popperInstance.disableEventListeners();
      }

      onMouseMove(lastMouseMoveEvent);
    },
    onTrigger: function onTrigger(instance, event) {
      __chunk_1.preserveInvocation(_onTrigger, instance.props.onTrigger, [instance, event]);
      wasTriggeredByFocus = event.type === 'focus';

      if (event instanceof MouseEvent) {
        lastMouseMoveEvent = event;
      }

      if (wasTriggeredByFocus && instance.popperInstance) {
        instance.popperInstance.reference = instance.reference;
      }

      if (!wasTriggeredByFocus && !(instance.state.isMounted && instance.props.followCursor === 'initial')) {
        addListener();
      }
    },
    onUntrigger: function onUntrigger(instance, event) {
      __chunk_1.preserveInvocation(_onUntrigger, instance.props.onUntrigger, [instance, event]); // The listener gets added in `onTrigger()`, but due to potential delay(s)
      // the instance made be untriggered before it shows. `onHidden()` will
      // therefore never be invoked.

      if (!instance.state.isVisible) {
        removeListener();
      }
    },
    onHidden: function onHidden() {
      __chunk_1.preserveInvocation(_onHidden, instance.props.onHidden, [instance]); // If scheduled to show before unmounting (e.g. delay: [500, 0]), the
      // listener should not be removed

      if (!instance.state.isScheduledToShow) {
        removeListener();
      }

      instance.popperInstance.options.placement = normalizedPlacement;
    }
  });

  instance.setProps = function (partialProps) {
    placement = partialProps.placement || placement;
    _onTrigger = partialProps.onTrigger || _onTrigger;
    _onUntrigger = partialProps.onUntrigger || _onUntrigger;
    _onMount = partialProps.onMount || _onMount;
    _onHidden = partialProps.onHidden || _onHidden;
    popperOptions = partialProps.popperOptions || popperOptions;
    originalSetProps(__chunk_1._extends({
      popperOptions: __chunk_1._extends({}, popperOptions, {
        onCreate: popperOnCreate
      })
    }, __chunk_1.removeProperties(partialProps, ['placement', 'onTrigger', 'onUntrigger', 'onMount', 'onHidden', 'popperOptions'])));
    setNormalizedPlacement();
    onMouseMove(lastMouseMoveEvent);
  };
}

exports.default = enhance;
exports.followCursor = withFollowCursor;
//# sourceMappingURL=tippy-extra-props.js.map
